/*
 *  avi_strl.c
 *  Derived from q4linux
 *
 *  Converts QuickTime movie files to AVI format
 *
 *  Modified by Benoit Joossen (July 2005).
 *  Company : Aero Quartet
 *
 *  You may copy, modify and redistribute this source file under
 *  the terms of the GNU Public License
 *
 */

#include "funcprotos.h"
#include "quicktime.h"
#include "glue.h"

// Update during close:
// length
// samples per chunk
#define JUNK_SIZE 0x1014



quicktime_strl_t* quicktime_new_strl()
{
	quicktime_strl_t *strl = calloc(1, sizeof(quicktime_strl_t));
	return strl;
}


void quicktime_init_strl(quicktime_t *file, 
	quicktime_audio_map_t *atrack,
	quicktime_video_map_t *vtrack,
	quicktime_trak_t *trak,
	quicktime_strl_t *strl)
{
	quicktime_atom_t list_atom, strh_atom, strf_atom;
	quicktime_atom_t junk_atom;
	int i;

/* Construct tag */
	if(vtrack)
	{
		strl->tag[0] = '0' + (trak->tkhd.track_id - 1) / 10;
		strl->tag[1] = '0' + (trak->tkhd.track_id - 1) % 10;
		strl->tag[2] = 'd';
		strl->tag[3] = 'c';
	}
	else
	if(atrack)
	{
		strl->tag[0] = '0' + (trak->tkhd.track_id - 1) / 10;
		strl->tag[1] = '0' + (trak->tkhd.track_id - 1) % 10;
		strl->tag[2] = 'w';
		strl->tag[3] = 'b';
		quicktime_stco_init(&(trak->mdia.minf.stbl.stco));
		quicktime_stco_init_common(file, &(trak->mdia.minf.stbl.stco));
	}


/* LIST 'strl' */
	quicktime_atom_write_header(file, &list_atom, "LIST");
	quicktime_write_char32(file, "strl");

/* 'strh' */
	quicktime_atom_write_header(file, &strh_atom, "strh");



/* vids */
	if(vtrack)
	{
		quicktime_write_char32(file, "vids");
		quicktime_write_char32(file, 
			trak->mdia.minf.stbl.stsd.table[0].format);
/* flags */
        quicktime_write_int32_le(file, 0);
/* priority */
        quicktime_write_int16_le(file, 0);
/* language */
        quicktime_write_int16_le(file, 0);
/* initial frame */
        quicktime_write_int32_le(file, 0);

/* framerate denominator */
/* framerate numerator */
quicktime_write_int32_le(file, 0);
quicktime_write_int32_le(file, 0);
// 01 Jan 2008 now written later

/* start */
        quicktime_write_int32_le(file, 0);
		strl->length_offset = quicktime_position(file);
/* length: fill later */
        quicktime_write_int32_le(file, 0); 
/* suggested buffer size */
        quicktime_write_int32_le(file, 0); 
/* quality */
        quicktime_write_int32_le(file, -1); 
/* sample size */
        quicktime_write_int32_le(file, 0); 
        quicktime_write_int16_le(file, 0);
        quicktime_write_int16_le(file, 0);
        quicktime_write_int16_le(file, trak->tkhd.track_width);
        quicktime_write_int16_le(file, trak->tkhd.track_height);
	}
	else
/* auds */
	if(atrack)
	{
        quicktime_write_char32(file, "auds");
        quicktime_write_int32_le(file, 0);
/* flags */
        quicktime_write_int32_le(file, 0);
/* priority */
        quicktime_write_int16_le(file, 0);
/* language */
        quicktime_write_int16_le(file, 0);
/* initial frame */
        quicktime_write_int32_le(file, 0);
		strl->samples_per_chunk_offset = quicktime_position(file);
/* samples per chunk */
        quicktime_write_int32_le(file, 0);
/* sample rate * samples per chunk  if uncompressed */
/* sample rate if compressed */
		if (isAudioCompressed) quicktime_write_int32_le(file, audioFactor); // 05 July 2007
		else quicktime_write_int32_le(file, trak->mdia.minf.stbl.stsd.table[0].sample_rate);//*quicktime_audio_bits(file, atrack)/8*trak->mdia.minf.stbl.stsd.table[0].channels);
/* start */
        quicktime_write_int32_le(file, 0);
		strl->length_offset = quicktime_position(file);
/* length, XXX: filled later */
        quicktime_write_int32_le(file, 0);
/* suggested buffer size */
		if (isAudioCompressed) quicktime_write_int32_le(file, 192); // XXXZ 05 July 2007
        else quicktime_write_int32_le(file, trak->mdia.minf.stbl.stsd.table[0].sample_rate*packetSize);
/* quality */
        quicktime_write_int32_le(file, -1);
/* sample size: 0 for compressed and number of bytes for uncompressed */
		strl->sample_size_offset = quicktime_position(file);
        quicktime_write_int32_le(file, 0);
        quicktime_write_int32_le(file, 0);
        quicktime_write_int32_le(file, 0);
			
	}
	quicktime_atom_write_footer(file, &strh_atom);







/* strf */
	quicktime_atom_write_header(file, &strf_atom, "strf");

	if(vtrack)
	{
/* atom size repeated */
		quicktime_write_int32_le(file, 40);
		quicktime_write_int32_le(file, trak->mdia.minf.stbl.stsd.table[0].width); // 06 July 2007
		quicktime_write_int32_le(file, trak->mdia.minf.stbl.stsd.table[0].height); // 06 July 2007
/* planes */
		quicktime_write_int16_le(file, 1);  
/* depth */
		quicktime_write_int16_le(file, 24); 
		quicktime_write_char32(file, 
			trak->mdia.minf.stbl.stsd.table[0].format);
		quicktime_write_int32_le(file, 
			trak->tkhd.track_width * trak->tkhd.track_height * 3);
		quicktime_write_int32_le(file, 0);
		quicktime_write_int32_le(file, 0);
		quicktime_write_int32_le(file, 0);
		quicktime_write_int32_le(file, 0);
	}
	else
	if(atrack)
	{
/* By now the codec is instantiated so the WAV ID is available. */
		int wav_id = 0x0;
		quicktime_codec_t *codec_base = atrack->codec;

		if(codec_base->wav_id)
			wav_id = codec_base->wav_id;
		quicktime_write_int16_le(file, 
			wav_id);
		quicktime_write_int16_le(file, 
			trak->mdia.minf.stbl.stsd.table[0].channels);
		quicktime_write_int32_le(file, 
			trak->mdia.minf.stbl.stsd.table[0].sample_rate);
/* bitrate in bytes */
		quicktime_write_int32_le(file, audioFactor); // XXXZZ
/* block align */
		if (isAudioCompressed && (!isSpecialADPCM)) quicktime_write_int16_le(file, 1); // 05 July 2007
		else quicktime_write_int16_le(file, packetSize);
/* bits per sample */
		if (isAudioCompressed && (!isSpecialADPCM)) quicktime_write_int16_le(file, 0); // 05 July 2007
		else if (isSpecialADPCM) quicktime_write_int16_le(file, 4); // 07 July 2007
		else quicktime_write_int16_le(file, quicktime_audio_bits(file, atrack));
		//	trak->mdia.minf.stbl.stsd.table[0].sample_size); 
		if (isAudioCompressed) {
			quicktime_write_int16_le(file, 12); // was not here
			quicktime_write_int16_le(file, quicktime_avg_chunk_samples(file, trak)); // was 0 . 16 Dec 2007
		}
		if (copySampleDescription) { // 06 Jan 2008
			int dSize = (** sdh).descSize;
			int offDescr = 0x64;
			unsigned char* dataPtr = (* sdh);
			//if (!isAudioCompressed) offDescr = 0x60; // 17 Feb 2008
			dataPtr += offDescr;
			quicktime_write_data(file, dataPtr, dSize - offDescr - 8);
		}	
	}

	quicktime_atom_write_footer(file, &strf_atom);




/* Junk is required in Windows. */
/* In Heroine Kernel it's padding for the super index */
	strl->indx_offset = quicktime_position(file);
	strl->padding_size = JUNK_SIZE;



	quicktime_atom_write_header(file, &junk_atom, "JUNK");
	for(i = 0; i < JUNK_SIZE; i += 4)
		quicktime_write_int32_le(file, 0);
	quicktime_atom_write_footer(file, &junk_atom);


/* Initialize super index */
	quicktime_init_indx(file, &strl->indx, strl);

	quicktime_atom_write_footer(file, &list_atom);
}



void quicktime_delete_strl(quicktime_strl_t *strl)
{
	quicktime_delete_indx(&strl->indx);
	free(strl);
}
/*
void quicktime_read_strl(quicktime_t *file, 
	quicktime_strl_t *strl, 
	quicktime_atom_t *parent_atom) */
